﻿using System.Collections.Generic;
using Clio.Utilities;
using ff14bot;
using ff14bot.Helpers;
using ff14bot.Navigation;
using ff14bot.Objects;
using TreeSharp;

namespace Gladiator
{
    public class Avoidance
    {
        private Dictionary<uint, TelegraphShape> spellShapes = new Dictionary<uint, TelegraphShape>();
        private bool isAvoiding;

        public Avoidance()
        {
            spellShapes = SpellShapeDatabase.SpellShapes;
        }

        public Composite GetAvoidanceBehavior()
        {
            return new Action(r => AvoidTelegraph());
        }

        private RunStatus AvoidTelegraph()
        {
            uint spellId;
            var enemyCasting = IsEnemyCastingTelegraph(out spellId);
            if (!enemyCasting && !isAvoiding) { return RunStatus.Failure; }

            if (!enemyCasting && isAvoiding)
            {
                Navigator.PlayerMover.MoveStop();
                if (Core.Me.CurrentTarget != null) { Core.Me.Face(Core.Me.CurrentTarget); }
                isAvoiding = false;
                return RunStatus.Failure;
            }

            if (isAvoiding) { return RunStatus.Success; }

            var safePosition = GetSafePosition(spellShapes[spellId]);
            Navigator.PlayerMover.MoveTowards(new Vector3(safePosition.X, safePosition.Y, 0));
            Logging.Write("Moving towards: " + safePosition);
            isAvoiding = true;

            return RunStatus.Success;
        }

        private bool IsEnemyCastingTelegraph(out uint spellId)
        {
            spellId = 0;
            var target = Core.Me.CurrentTarget as BattleCharacter;

            if (target == null || !target.IsCasting || target.SpellCastInfo == null) { return false; }

            spellId = target.SpellCastInfo.ActionId;
            return spellShapes.ContainsKey(spellId);
        }

        private static Vector3 GetSafePosition(TelegraphShape shape)
        {
            switch (shape)
            {
                case TelegraphShape.Cone:
                    return GetPerpendicularPosition();
                case TelegraphShape.Rectangle:
                    return GetPerpendicularPosition();
                default:
                    return Core.Me.Location;
            }
        }

        private static Vector3 GetPerpendicularPosition()
        {
            var enemyPos = Core.Me.CurrentTarget.Location;
            Logging.Write("Enemy Position: " + enemyPos);
            var myPos = Core.Me.Location;
            Logging.Write("My Position: " + myPos);

            Core.Me.Face(enemyPos);

            var normalEnemyPos = new Vector3(enemyPos.X - myPos.X, enemyPos.Y - myPos.Y, myPos.Z);
            Logging.Write("Normal Enemy Pos:" + normalEnemyPos);
            var perpendicular = new Vector3(-normalEnemyPos.Y, normalEnemyPos.X, 0);
            Logging.Write("Perpendicular: " + perpendicular);

            return perpendicular + myPos;
        }
    }
}