﻿using ff14bot;
using ff14bot.AClasses;
using ff14bot.Behavior;
using ff14bot.Helpers;
using ff14bot.Navigation;
using TreeSharp;

namespace Gladiator
{
    public class PaladinBot : BotBase
    {
        private Composite root;

        public override string Name
        {
            get { return "Paladin Bot"; }
        }

        public override bool IsAutonomous
        {
            get { return false; }
        }

        public override bool WantButton
        {
            get { return false; }
        }

        public override void Start()
        {
            Navigator.NavigationProvider = new AStarNavigator();
            Navigator.PlayerMover = new SlideMover();
        }

        public override PulseFlags PulseFlags
        {
            get { return PulseFlags.All; }
        }

        public override bool RequiresProfile
        {
            get { return false; }
        }

        public override Composite Root
        {
            get { return root ?? (root = CreateRotation()); }
        }

        private static Composite CreateRotation()
        {
            var avoidance = new Avoidance();
            var avoid = avoidance.GetAvoidanceBehavior();

            var riotBlade = new Decorator(req => RiotBladeCheck,
                Casting.Cast("Riot Blade"));

            var rageOfHalone = new Decorator(req => RageOfHaloneCheck,
                Casting.Cast("Rage of Halone"));

            var savageBlade = new Decorator(req => SavageBladeCheck,
                Casting.Cast("Savage Blade"));

            var fastBlade = Casting.Cast("Fast Blade");

            var rotation = new PrioritySelector(avoid);

            //var rotation = new PrioritySelector(avoid, riotBlade, rageOfHalone,
            //    savageBlade, fastBlade);

            var rotationCheck = new Decorator(req => Core.Player.InCombat && Targeting.HasCombatTarget, rotation);

            return new LockSelector(rotationCheck);
        }

        public static bool RiotBladeCheck
        {
            get { return Casting.WasLastSpell("Fast Blade") && Core.Player.CurrentManaPercent < 30; }
        }

        public static bool RageOfHaloneCheck
        {
            get { return Casting.WasLastSpell("Savage Blade"); }
        }

        public static bool SavageBladeCheck
        {
            get { return Casting.WasLastSpell("Fast Blade"); }
        }

        private class LockSelector : PrioritySelector
        {
            public LockSelector(params Composite[] children)
                : base(children) { }

            public override RunStatus Tick(object context)
            {
                using (Core.Memory.AcquireFrame())
                {
                    return base.Tick(context);
                }
            }
        }
    }
}