﻿using ff14bot;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Objects;
using TreeSharp;

namespace Gladiator
{
    public static class Targeting
    {
        public static Character CastTarget
        {
            get
            {
                var spellCastInfo = Core.Player.SpellCastInfo;
                if (spellCastInfo == null) { return null; }

                var castTargetId = spellCastInfo.TargetId;
                if (castTargetId == GameObjectManager.EmptyGameObject) { return null; }

                return GameObjectManager.GetObjectByObjectId(castTargetId) as Character;
            }
        }

        public static Composite EnsureCastTarget()
        {
            return new Decorator(req => CastTargetIsDead, 
                Casting.StopCasting);
        }

        private static bool CastTargetIsDead
        {
            get
            {
                var target = CastTarget;
                return target != null && target.IsDead;
            }
        }

        public static GameObject Target
        {
            get { return Core.Player.CurrentTarget; }
        }

        public static Character CharTarget
        {
            get { return Core.Player.CurrentTarget as Character; }
        }

        public static bool HasCombatTarget
        {
            get
            {
                var combatTarget = Core.Me.CurrentTarget as BattleCharacter;
                return combatTarget != null && combatTarget.InCombat;
            }
        }

        public static bool IsNearTarget(Character character, float range)
        {
            var target = Target;
            if (target == null) { return false; }
            return character.Location.Distance3D(target.Location) <= range;
        }
    }
}